#
# XMLWriter.py
#
# Generate arbitrary XML files
#
# (c) 2004, Davyd Madeley <davyd@ucc.asn.au>
#

import time

class PlanetItem:
	def __init__(self, blog, item):
		self.itemTitle	= item.itemTitle
		self.itemURL	= item.itemURL
		self.itemDate	= item.itemDate
		self.blogTitle	= blog.blogTitle
		self.blogURL	= blog.blogURL
		self.imageURL	= blog.imageURL
		self.imageLink	= blog.imageLink
		self.contents	= item.contents

class PlanetDate:
	def __init__(self, date):
		self.planetDate	= date
		self.items	= []

class Planet:
	def __init__(self, bloglist):
		self.__bloglist__	= bloglist
		self.__tainted__	= True
		self.dates		= []
		
	def append(self, blog):
		self.__bloglist__.append(blog)
		self.__tainted__	= True
	
	def __getNext__(self, bloglist):
		"Returns a PlanetItem reaped from a bloglist"
		latestTime		= 0
		holdingBlog		= None
		for blog in bloglist:
			if len(blog.items) > 0 and blog.items[0].itemDate > latestTime:
				latestTime	= blog.items[0].itemDate
				holdingBlog	= blog
		if holdingBlog == None:
			return None
		item	= holdingBlog.items.pop(0)
		return PlanetItem(holdingBlog, item)
	
	def sort(self):
		if self.__tainted__:
			# we need to sort the blogs into a single news feed
			# copy the bloglist to a working symbol
			bloglist	= self.__bloglist__ + []
			lastDate	= -1
			workingDate	= None
			self.dates	= []
			while True:
				lastItem	= self.__getNext__(bloglist)
				if lastItem == None:
					break
				# this checks to see if it's a new day
				if time.localtime(lastItem.itemDate)[6] != lastDate:
					lastDate	= time.localtime(lastItem.itemDate)[6]
					workingDate	= PlanetDate(lastItem.itemDate)
					self.dates.append(workingDate)
				# append the item to the current date
				workingDate.items.append(lastItem)	
			self.__tainted__	= False
		return self.dates
			

class XMLWriter:
	def __init__(self, bloglist):
		self.blogs	= bloglist
		self.planet	= Planet(bloglist)
		self.items	= self.planet.sort()
	
	def write(self, doctype):
		# doctype should be something like XHTMLWriter.XHTMLWriter
		writer		= doctype(self.items)
		writer.parent	= self
		output		= writer.write()
		return output

